/*=========================================================================
 *
 *  Copyright Insight Software Consortium
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0.txt
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *=========================================================================*/
#ifndef itkLogSigmoidTransferFunction_h
#define itkLogSigmoidTransferFunction_h

#include "itkTransferFunctionBase.h"

namespace itk
{
namespace Statistics
{
/** \class LogSigmoidTransferFunction
 * \brief This is the itkLogSigmoidTransferFunction class.
 *
 * \ingroup ITKNeuralNetworks
 */

template<typename TScalar>
class ITK_TEMPLATE_EXPORT LogSigmoidTransferFunction : public TransferFunctionBase<TScalar>
{
public:

  /** Standard class type alias. */
  using Self = LogSigmoidTransferFunction;
  using Superclass = TransferFunctionBase<TScalar>;
  using Pointer = SmartPointer<Self>;
  using ConstPointer = SmartPointer<const Self>;

  using ScalarType = TScalar;
  using RealType = typename NumericTraits< ScalarType >::RealType;

  /** Run-time type information (and related methods). */
  itkTypeMacro(LogSigmoidTransferFunction, TransferFunctionBase);

  /** Method for creation through the object factory. */
  itkNewMacro(Self);

  /** Evaluate at the specified input position */
  TScalar Evaluate(const ScalarType& input) const override;

  /** Evaluate the derivative at the specified input position */
  TScalar EvaluateDerivative(const ScalarType& input) const override;

protected:

  LogSigmoidTransferFunction();
  ~LogSigmoidTransferFunction() override;

  /** Method to print the object. */
  void PrintSelf( std::ostream& os, Indent indent ) const override;
};

} // end namespace Statistics
} // end namespace itk

#ifndef ITK_MANUAL_INSTANTIATION
#include "itkLogSigmoidTransferFunction.hxx"
#endif


#endif
