/*=========================================================================
 *
 *  Copyright Insight Software Consortium
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0.txt
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *=========================================================================*/
#ifndef itkRadialBasisFunctionBase_h
#define itkRadialBasisFunctionBase_h

#include "itkFunctionBase.h"
#include "itkArray.h"

namespace itk
{
namespace Statistics
{
/** \class RadialBasisFunctionBase
 * \brief This is the itkRadialBasisFunctionBase class.
 *
 * \ingroup ITKNeuralNetworks
 */

template<typename ScalarType>
class RadialBasisFunctionBase : public FunctionBase<ScalarType,ScalarType>
{
public:
  ITK_DISALLOW_COPY_AND_ASSIGN(RadialBasisFunctionBase);

  /** Standard class type alias. */
  using Self = RadialBasisFunctionBase;
  using Superclass = FunctionBase<ScalarType,ScalarType>;
  using Pointer = SmartPointer<Self>;
  using ConstPointer = SmartPointer<const Self>;

  /** Run-time type information (and related methods). */
  itkTypeMacro(RadialBasisFunctionBase, FunctionBase);

  /** Input/Output types */
  using ArrayType = Array<ScalarType>;

  ///** Evaluate at the specified input position */
  ScalarType Evaluate(const ScalarType& input) const override =0;

  /** Evaluate the derivative at the specified input position */
  virtual ScalarType EvaluateDerivative(const ScalarType& dist, const ArrayType& input,
                                                    char mode,int element_id=0) const = 0;

  itkSetMacro(Radius,ScalarType);
  itkGetConstMacro(Radius, ScalarType );

  itkSetMacro(Center,ArrayType);
  itkGetConstMacro(Center, ArrayType );

protected:

  RadialBasisFunctionBase()
    {
    m_Radius = 0;
    }
  ~RadialBasisFunctionBase() override {};

  /** Method to print the object. */
  void PrintSelf( std::ostream& os, Indent indent ) const override
    {
    os << indent << "RadialBasisFunctionBase(" << this << ")" << std::endl;
    Superclass::PrintSelf( os, indent );
    }

private:

  ArrayType  m_Center;
  ScalarType m_Radius;

};

} // end namespace Statistics
} // end namespace itk

#endif
